c |$Id: bimg.f,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $              |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://173.255.241.228                | 
c | PROGRAM NAME: bimg.f                                       |
c | DIRECTORY: bsu-3.0.3/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |         CROSS CORRELATES  ONE TRACE IN BSEGY DATA WITH     |
c |         OTHER TRACES                                       |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2021 Paul Michaels                          |
c |  <paulmichaels@boisestate.edu>                             |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-3.0.3/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     XCOR    Computes cross or auto correlation, 2 sided    |
c |                                                            |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=65536)
c
      real*4            s1(ndim),s2(ndim),s3(ndim),s4(ndim),s5(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, outfil2
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      iunit4=4
      icall1=0
      icall2=0
      icall3=0
      icall4=0
c
c...define application prefix
      write(aplc,'(''bimg'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +t1,t2,tlagmx,jnear,jfar)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)

c...determine average group interval
      icall1 = 0
      jrec1 = 1
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec1,iexit)
      call findxyz(sx,sy,sz,sd,rx,ry,rz,frange1,hd)
c     redefine frange to a spatial coordinate (not source to receiver)
      frange1=sqrt(rx*rx+ry*ry+rz*rz)

      jrec2 = 5
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec2,iexit)
      call findxyz(sx,sy,sz,sd,rx,ry,rz,frange2,hd)
c     redefine frange to a spatial coordinate (not source to receiver)
      frange2=sqrt(rx*rx+ry*ry+rz*rz)

      if (ntrace.lt.5) then 
      write(*,*) "Group interval determination needs at least 5 traces"
      stop
      endif
      igroupint = int(abs(frange2-frange1)/float(jrec2-jrec1) +.5)

c...set number of traces to available to an even number, N
      N = (ntrace/2)*2

c...check validity of parameters
      if (jnear .lt.1 ) then 
      jnear = 1
      endif
      if (jfar.gt.N-1) then
      jfar=N-1
      endif 

c...set number of traces to use to an even number, NU
      NU=jfar-jnear + 1
      NU = (NU/2)*2
c
c...check validity of parameters
      tmax1=fsamin*(npts-1)
      if(t1.lt.0) t1=0
      if(t2.gt.tmax1) t2=tmax1
      tzero=tlagmx/2

c...compute start and end samples (using xcor.f rules)
      is1=int(t1/fsamin + 0.5) + 1
      is2=int(t2/fsamin + 0.5) + 1
      ismx=int(tlagmx/fsamin)
      izero=int(tzero/fsamin)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
      write(outfil2,'(''BIMG'',a,''.seg'')') infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,t1,t2,tlagmx,tzero,frange1,frange2,igroupint,jnear,jfar)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
      open(10,file=outfil2,access='direct',recl=240)
      close(10,status='delete')
c
      icall2=0
      icall3=0
      ntr=0

c...number of samples to be output
      npts3=tlagmx/fsamin
c
c... loop over correlation trace offsets, k
c      do 100 k=1,N/2 
c      do 100 k=1,N-1 
      kcount = 0
      do 100 k=jnear,jfar
      kcount = kcount + 1

c...clear stack trace
      do 698 jt=1,npts3
      s4(jt)=0.0
      s5(jt)=0.0
  698 continue

c... k associated with offset instance
      ioffset = k * igroupint

c      do 700 itrace=1,(N-k)
      do 700 itrace=1,N

c... read trace to correlate with, itrace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,itrace,iexit)
c
      do 37 jt=1,npts
   37 s2(jt)=s1(jt)
c
c...compute trace number to correlate against for this offset
      if((itrace+k) .le. N) then
      jrec = itrace + k
      else
      goto 700
      endif
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      call xcor(npts,t1,t2,tlagmx,tzero,fsamin,s2,s1,s3)

c     quality check write below (uncomment for a big detailed listing)
c     write(io11,'("Correlating: ",i5," with " i5)') itrace, jrec

c...stack result for this offset
      do 699 jt=1,npts3
      s4(jt) = s4(jt) + s3(jt) 
  699 continue 

      jt5=1
      do 701 jt=izero,1,-1
      s5(jt5)=s5(jt5) + s3(jt)/2. + s3(jt5-1+izero)/2.
      jt5 = jt5 + 1
  701 continue
      npts4=jt5-1

c...end correlating trace loop for this offset
  700 continue

c...reset header values
      ishlo = 0
      ishla = 0
      if(smul2.lt.0) then
      irlo = ioffset*abs(smul2)
      else
      irlo = ioffset/smul2
      endif
      irla = 0
      cunits = 1
c
c...output trace
      CALL BSEGOUT(iunit3,
     +icall3,npts3,s4,fsamin,outfil,kcount)
c
      CALL BSEGOUT(iunit4,
     +icall4,npts4,s5,fsamin,outfil2,kcount)
c
c...display progress....
      call pltbar(bar,invbar,ibar,kcount,NU)
  100 continue

c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
c     stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://173.255.241.228                | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +t1,t2,tlagmx,jnear,jfar)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6,arg7
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2021  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bimg: Cross-correlate traces to form     |'
      write(*,*) '| a seismic image for passive sources      |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bimg infile1 t1 t2 tlagmx jnear jfar '
      write(*,*) ' '
      write(*,*) 'infile1: name of input file #1 '
      write(*,*) 't1:     =start time of cross correlation gate (sec.)'
      write(*,*) 't2:     =end time of cross correlation gate (sec.) '
      write(*,*) 'tlagmx: =maximum cross correlation lag time (sec.)'
      write(*,*) 'jnear:  = near correlation offset (trace spacings)'
      write(*,*) 'jfar:   = far correlation offset (trace spacings) '
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file #1'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,'(''enter t1=start time correlation gate(float,sec)'')')
      read(*,*) t1
      else
      call getarg(2,arg2)
      read(arg2,'(f10.4)') t1
      endif
      if(nargsx.lt.(4+karg)) then
      write(*,'(''enter t2 =end time correlation gate(float, sec)'')')
      read(*,*) t2
      else
      call getarg(3,arg3)
      read(arg3,'(f10.4)') t2
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,'(''enter tlagmx = maximum lag in (float,seconds)'')')
      read(*,*) tlagmx
      else
      call getarg(4,arg4)
      read(arg4,'(f10.4)') tlagmx
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''enter jnear =near correl. offset(int, traces)'')')
      read(*,*) jnear
      else
      call getarg(5,arg5)
      read(arg5,'(i10)') jnear
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,'(''  enter jfar = far correl. offset(int, traces)'')')
      read(*,*) jfar
      else
      call getarg(6,arg6)
      read(arg6,'(i10)') jfar
      endif

      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://173.255.241.228                | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,t1,t2,tlagmx,tzero,frange1,frange2,igroupint,jnear,jfar)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2021  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(11,'('' Program bimg'')')
      write(11,'('' Input File #1: '',a)') infil
      write(11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Trace 1 Coordinate='',f10.4)') frange1 
      write(io11,'(''  Trace 5 Coordinate='',f10.4)') frange2
      write(io11,'(''  Average Group Interval='',i5)') igroupint
    
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   t1 =  '',f10.2)') t1  
      write(io11,'(''   t2 =  '',f10.2)') t2  
      write(io11,'(''   tlagmx='',f10.2)') tlagmx
      write(io11,'(''   Zero lag time on output='',f10.4)') tzero
      write(io11,'(''jnear = near correl. offset (traces)'',i10)') jnear
      write(io11,'(''jfar  = far  correl. offset (traces)'',i10)') jfar
      write(io11,'(''    '')')
      inear=igroupint*jnear
      write(io11,'('' near correlation offset (meters) ='',i10)') inear
      ifar=igroupint*jfar
      write(io11,'('' far  correlation offset (meters) ='',i10)') ifar
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

