// |$Id: WriteSEG2.c,v 1.1.1.1 2024/10/07 17:24:38 pm Exp $     |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://173.255.241.228              | 
   | FUNCTION NAME: fnorm.c                                     |
   | DIRECTORY: bsu-3.0.3/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2021 Paul Michaels                          |
   |  <paulmichaels@boisestate.edu>                             |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
*/
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <float.h>

	void writestring(char STRING[], char RES, char LTC,FILE *h1);



//=========================WriteSEG2=============================================	
int	WriteSEG2(int npts, int nchnl, float fsamin, int s1[], float DELAY,
	char rawrecord[], int stack, char source[], char phones[], double DSF,
	short int AIF, short int AIS, short int FLC, short int SLC, short int FHC,
	short int SHC, short int GIDB,
	short nSP, float sx, float sy, float sz, float rx[], float ry[], float rz[],
	char TIME[], char DATE[], char STRING[]) 
{
//...declare variables
	time_t            start;
	struct tm        *ptr;
//C++ requires char const, not just char
	char const *mon[12]={"Jan","Feb","Mar","Apr","May","Jun",
        "Jul","Aug","Sep","Oct","Nov","Dec"};

	FILE    *h1;
	char ofile[13] ;
	int	i, k;
	unsigned short int ID = 0x3a55;  				//bytes 0 1
	unsigned short int REV = 1;					//bytes 2 3
	/* Note: 8 channel recorder, but allow max 10 channels for pointers space
	This will permit some zeros for buffer to free format section */
	unsigned short int M ;  // size of trace point block NCH*4  //bytes 4 5
	unsigned short int NCH   ; // number of channels 		//bytes 6 7
	char LENST = 0x01; // length of string terminator  (bytes)	//byte  8
	char ST = 0; //string terminator byte				//bytes 9 10
	char LT = 0x01; //length of line terminator character (bytes)	//byte  11
	char LTC = 0x0a; // line terminator				//byte  12
	char RES = 0x00; //reserved byte				//bytes 14-31
	int ZERO4 = 0;   //4 byte zero
	short int ZERO2 = 0;   //2 byte zero
	unsigned short int TDBID = 0x4422;
	unsigned long int TPB, LOC1, LOC2, EFDB, EDB;
	int CHNL ;
//	int stack ;
	int FG; //fixed gain variable
//	double DSF; //descale factor


//...set parameters
	char lineid[] = "0001";
	sprintf(ofile,"%s",rawrecord);

	char typerecvr[30];
	char typesrc[30];
	switch (source[0]) {
		case 'H':
		sprintf(typesrc,"HAMMER");
		break;
		case 'W':
		sprintf(typesrc,"WEIGHT_DROP");
		break;	
		case 'D':
		sprintf(typesrc,"DYNAMITE");
		break;	
		case 'G':
		sprintf(typesrc,"GUN");
		break;	
		case 'V':
		sprintf(typesrc,"VIBRATOR");
		break;	
		case 'S':
		sprintf(typesrc,"SYNTHETIC");
		break;	
		default:
		sprintf(typesrc,"HAMMER");
		}

	NCH = nchnl; 
	CHNL = 0;
	M=NCH*4;  // M= NCH * 4 = 192 if NCH max = 48

	for (i=0; i<nchnl; i++) {
	TIME[i] = '\0';
	DATE[i] = '\0'; }


//...open output file
        if( (h1=fopen(ofile,"wb"))==NULL )
	{
	fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
	return(1);
	}

//...GET TIME AND DATE
      time(&start);
      ptr=localtime(&start);
      sprintf(TIME,"  ACQUISITION_TIME %2.2u:%2.2u:%2.2u  ",
         (*ptr).tm_hour,(*ptr).tm_min,(*ptr).tm_sec);
      sprintf(DATE,"  ACQUISITION_DATE %2.2d/%3s/%4.4d  ",
         (*ptr).tm_mday,mon[(*ptr).tm_mon],(*ptr).tm_year+1900);
	
//...FILE DESCRIPTOR BLOCK
	fwrite(&ID,sizeof(short int),1,h1); //bytes 0,1 ID
	fwrite(&REV,sizeof(short int),1,h1); //bytes 2,3 revision number
	fwrite(&M,sizeof(short int),1,h1);//bytes 4,5 size trace pointer block
	fwrite(&NCH,sizeof(short int),1,h1); //bytes 6,7 number of channels
	fwrite(&LENST,sizeof(char),1,h1);  // byte 8 size of ST
	fwrite(&ST,sizeof(char),1,h1); //byte 9   string term char used
	fwrite(&ST,sizeof(char),1,h1); //byte 10  2nd string term char
	fwrite(&LT,sizeof(char),1,h1); //byte 11  size LTC
	fwrite(&LTC,sizeof(char),1,h1); //byte 12 line term char
	fwrite(&ST,sizeof(char),1,h1); //byte 13 2nd line term char not used
	for (i=14; i<32; i++) {
	fwrite(&RES,sizeof(char),1,h1); }

//...TRACEPOINTER SUBBLOCK
//	TPB = ftell(h1);    //capture start of block to come back to later
	TPB = (long int) 0x20;
	for (i=0; i<nchnl; i++) {
	fwrite(&ZERO4,sizeof(int),1,h1);} //zero out pointers to start 

//...Write Date of acquisition
	LOC1=ftell(h1);
	writestring(DATE,RES,LTC,h1);

//...Write Time of acquisition
	writestring(TIME,RES,LTC,h1);

//...Write Instrument 
	sprintf(STRING,"  INSTRUMENT PM_CONVERT BSEGY TO SEG-2 ");
	writestring(STRING,RES,LTC,h1);

//...Write Instrument 
	char UNITS[nchnl];
	sprintf(UNITS,"  UNITS METERS  ");
	writestring(UNITS,RES,LTC,h1);

//...capture end of free format location
	EFDB=ftell(h1) - 1 ;
//	printf("location end of FDB = %0lx \n",EFDB);

	EDB = EFDB ;  //not really end of data block first time
			// EDB is also start of TDB later on
	
/* START OF TRACE DESCRIPTOR BLOCK ===================================== */

	for (k=0; k<nchnl; k++) {

//return to trace pointer block start and write LOC2
	fseek(h1,TPB+k*4,SEEK_SET);
	LOC2= EDB + 1 ;  // add one to point beyond end of FDB or EDB
	fwrite(&LOC2,sizeof(int),1,h1);


//...Write Trace Descriptor block
	fseek(h1,LOC2,SEEK_SET);  //return to start of trace descriptor block
//printf("%08lx \n",LOC2);
	fwrite(&TDBID,sizeof(short int),1,h1);  //write ID code for TDB

	LOC1=ftell(h1);  //start of trace descriptor block just after ID
	short int TDBS = ZERO2;
	fwrite(&TDBS,sizeof(short int),1,h1); //write zero temporary for block size
	int SB = 4*npts;	//size of data block in bytes
	fwrite(&SB,sizeof(int),1,h1); //size of data block in bytes
	int NS = npts; 		// number of samples in a data block
	fwrite(&NS,sizeof(int),1,h1); //number of samples in data block
	char DFC = 0x02; // Data Format Code for 32 bit integer		
	fwrite(&DFC,sizeof(char),1,h1);
	/* Reserved */

/* NOTE Strings must start with 2 spaces to allow offset to next replacement int
        and must end with 2 spaces to allow for \0 and \0a replacement (eofs eofl) */
	for (i=13; i<32; i++) {
	fwrite(&RES,sizeof(char),1,h1);
	 }
	sprintf(STRING ,"  CHANNEL_NUMBER %d  ",CHNL+k);
	writestring(STRING ,RES,LTC,h1);

	sprintf(STRING,"  ALIAS_FILTER %d %d  ",AIF,AIS);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  HIGH_CUT_FILTER %d %d  ",FHC,SHC);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  LOW_CUT_FILTER %d %d  ",FLC,SLC);
	writestring(STRING,RES,LTC,h1);

	// switching to an argument of call, base on bsegy data
	//FG = 19;  //gain in dBv (change if your instrument different)
	FG = (int) GIDB;
	sprintf(STRING,"  FIXED_GAIN %d  ",FG);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  DELAY %.2f  ",DELAY);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  LINE_ID %s  ",lineid);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  RAW_RECORD %s  ",rawrecord);
	writestring(STRING,RES,LTC,h1);

	switch (phones[k]) {
		case 'V':
		sprintf(typerecvr,"VERTICAL_GEOPHONE");
		break;
		case 'T':
		sprintf(typerecvr,"T_HORIZONTAL_GEOPHONE");
		break;
		case 'R':
		sprintf(typerecvr,"R_HORIZONTAL_GEOPHONE");
		break;
		default:
		sprintf(typerecvr,"VERTICAL_GEOPHONE");
		}

	sprintf(STRING,"  RECEIVER %s  ",typerecvr);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  SOURCE %s  ",typesrc);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  SOURCE_STATION_NUMBER %4.4d  ",nSP);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  RECEIVER_LOCATION %.2f %.2f %.2f  ",rx[k],ry[k],rz[k]);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  SOURCE_LOCATION %.2f %.2f %.2f  ",sx,sy,sz);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  SAMPLE_INTERVAL %.6f  ",fsamin);
	/* Arduino A/D -->SEIS16v2 s1[] is in microvolts, SEG-2 Standard 
	requires a descaling factor applied to the SEG-2 data which will
	yield mvolts */
	writestring(STRING,RES,LTC,h1); 

//Descale Factor based on dBv FG fixed gain, replace as an argument 	
//   	DSF = 1.000 / (1000. * pow((double) 10., (double) ((float) FG)/20.0));
	sprintf(STRING,"  DESCALING_FACTOR %14.7E  ",DSF);
	writestring(STRING,RES,LTC,h1);

	sprintf(STRING,"  STACK %d  ",stack);
	writestring(STRING,RES,LTC,h1);

	LOC2 = ftell(h1);
	TDBS = LOC2 - LOC1 + 2 ;  // add 2 bytes to include ID 0x4422

//determine padding if needed for Trace Descriptor Block TDB
//...Size, TDBS must be divisible by 4
	int pad,p1;
	p1=TDBS % 4;
	if ( p1 != 0) {
		pad = 4-p1;
		TDBS = TDBS + pad;
	} //endif
//printf("pad = %d \n",pad);
//	printf("Trace Descriptor Block Size (bytes) %d \n",TDBS);	
	fseek(h1,LOC1,SEEK_SET);
	fwrite(&TDBS,sizeof(short int),1,h1); //write block size
	fseek(h1,LOC2,SEEK_SET);

	if ( p1 != 0) {
		for (i=0; i<pad; i++) fwrite(&ST,sizeof(char),1,h1);
	}
//	long LOC3;
//	LOC3=ftell(h1);
//	printf("LOC3 = %8lx \n",LOC3);

//...WRITE DATA
	int *pr;
	pr = &s1[0] + k;
	/*NOTE: DC level is removed from data since pr changes signal
	 on exit:  *pr = *pr - ybase[k] changes s1, a pointer */
	for (i=0; i<npts; i=i+1) {
//	*pr = *pr - ybase[k]; 
	fwrite(pr,sizeof(int),1,h1);
	pr = pr + nchnl;
	} //end of data write
	
//...save end of data block
	EDB = ftell(h1) - 1; //the last byte of 4byte integer

	} // next k chanl
	fclose(h1);	
	return(0);
} //end WriteSEG2

void writestring(char STRING[], char RES, char LTC,FILE *h1)
{
	int nbytes, nbytesf;
	nbytes = fprintf(h1,"%s",STRING);
	fseek(h1,-nbytes,SEEK_CUR);
	nbytesf = nbytes + 1  ;  //replace 2 blanks at begin with stringlength
//		printf("bytes %d \n",(int) nbytes);
	fwrite(&nbytesf,sizeof(short int),1,h1); //write number of bytes forward
						//to next 2byte forward of next
	fseek(h1,nbytes-3,SEEK_CUR);
	fwrite(&RES,sizeof(char),1,h1); //replace 2nd last char with \0 endofstring 
	fwrite(&LTC,sizeof(char),1,h1); //replace last char with 0xa endofstring 
}
